{ This demonstration program shows how to use the function interpreter that's implemented
  in ChroMathsome as well as some of the other calculus functions.
  The reader may do with this code whatever he/she desires.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: April 12 '97

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
                }


unit Funcan1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls;

type
  TForm1 = class(TForm)
    RadioGroup1: TRadioGroup;
    FunctionEdit: TEdit;
    Label1: TLabel;
    VariableEdit: TEdit;
    Label2: TLabel;
    UpBoundEdit: TEdit;
    LowBoundEdit: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    ValueEdit: TEdit;
    Label5: TLabel;
    StepsEdit: TEdit;
    Label6: TLabel;
    ResultEdit: TEdit;
    Label7: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
        
  private
    { Private declarations }
    function IsDataOk(i: integer): boolean;

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Calculus;

{ Check if user has provided sufficient data.
  Of course, user should always specify function & independent variable!  }
function TForm1.IsDataOk(i: integer): boolean;
var a, b: extended;
    int: integer;
begin
     IsDataOk:= true;
     case i of
     1:begin // function value
        try
        a:= StrToFloat(ValueEdit.Text);
        except on EConvertError do IsDataOk:= false;
        end;
       end;
     2:begin // root
        try
        a:= StrToFloat(ValueEdit.Text);
        int:= StrToInt(StepsEdit.Text);
        except on EConvertError do IsDataOk:= false;
        end;
        end;
     3:begin // derivatives
        try
        a:= StrToFloat(ValueEdit.Text);
        b:= StrToFloat(StepsEdit.Text);
        except on EConvertError do IsDataOk:= false;
        end;
       end;
     4:begin // Integral
        try
        a:= StrToFloat(UpBoundEdit.Text);
        b:= StrToFloat(LowBoundEdit.Text);
        int:= StrToInt(StepsEdit.Text);
        except on EConvertError do IsDataOk:= false;
        end;
       end;
     5:begin //  Series. Data should be of type (long) integer but we'll ignore this for now
        try
        a:= StrToFloat(UpBoundEdit.Text);
        b:= StrToFloat(LowBoundEdit.Text);
        except on EConvertError do IsDataOk:= false;
        end;
       end;
     end; // case
end;

{ Reset editboxes. }
procedure TForm1.BitBtn2Click(Sender: TObject);
begin
     FunctionEdit.Text:= '0';
     VariableEdit.Text:= '0';
     UpBoundEdit.Text:= '0';
     LowBoundEdit.Text:= '0';
     ValueEdit.Text:= '0';
     StepsEdit.Text:= '0';
     ResultEdit.Text:= '0';
end;

procedure TForm1.BitBtn3Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
var Value, UpBound, LowBound: extended;
    UserFunction, SubFunction: string;
    IndepVar: char;
    Steps, LowI, UpI: integer;
begin
     UserFunction:= FunctionEdit.Text;
     IndepVar:= VariableEdit.Text[1];
     case RadioGroup1.ItemIndex of
     0:begin // function value
        if IsDataOk(1) = false then
        ShowMessage('Error in input data!')
        else
        begin
        Value:= StrToFloat(ValueEdit.Text);
        ResultEdit.Text:= FloatToStr(FunctionValue(UserFunction, IndepVar, Value));
        end;
       end;
     1:begin // root
        if IsDataOk(2) = false then
        ShowMessage('Error in input data!')
        else
        begin
        Value:= StrToFloat(ValueEdit.Text); // initial guess
        Steps:= StrToInt(StepsEdit.Text);
        ResultEdit.Text:= FloatToStr(FunctionRoot(UserFunction, IndepVar, Value, Steps));
        end;
        end;
     2,3,4:begin // derivatives
        if IsDataOk(3) = false then
        ShowMessage('Error in input data!')
        else
        begin
        Value:= StrToFloat(ValueEdit.Text); // initial guess
        UpBound:= StrToFloat(StepsEdit.Text);
        UpBound:= 1/UpBound;
        case RadioGroup1.ItemIndex of
        2:ResultEdit.Text:= FloatToStr(FirstDerivative(UserFunction, IndepVar, Value, UpBound));
        3:ResultEdit.Text:= FloatToStr(SecondDerivative(UserFunction, IndepVar, Value, UpBound));
        4:ResultEdit.Text:= FloatToStr(ThirdDerivative(UserFunction, IndepVar, Value, UpBound));
        end; // case
        end;
       end;
     5:begin // defined (single) integral
        if IsDataOk(4) = false then
        ShowMessage('Error in input data!')
        else
        begin
        UpBound:= StrToFloat(UpBoundEdit.Text); // interval bounds
        LowBound:= StrToFloat(LowBoundEdit.Text);
        Steps:= StrToInt(StepsEdit.Text);
        ResultEdit.Text:= FloatToStr(DefIntegral(UserFunction, IndepVar, UpBound, LowBound, Steps));
        end;
        end;
     6:begin // Series sum
        if IsDataOk(5) = false then
        ShowMessage('Error in input data!')
        else
        begin
        UpI:= StrToInt(UpBoundEdit.Text); // bounds
        LowI:= StrToInt(LowBoundEdit.Text);
        ResultEdit.Text:= FloatToStr(SeriesSum(UserFunction, IndepVar, UpI, LowI));
        end;
        end;
     7:begin // Composite function
        SubFunction:= ValueEdit.Text;
        FunctionComposition(UserFunction, IndepVar, SubFunction);
        ResultEdit.Text:= UserFunction;
        end;
     end; // case
end;


end.
